@@ -21,4 +21,8 @@ pub mod frame;
 pub mod balance;
 pub mod balance_manager;
 
-pub use balance_manager::*;
\ No newline at end of file
+pub use balance_manager::*;
+
+
+pub use dark_std::errors::Error;
+pub use dark_std::errors::Result;
\ No newline at end of file
